@@ -14,6 +14,9 @@
 #'
 #' @return a matrix of period-region summary of the Horvitz-Thompson direct estimates by region and time period specified in the argument, the standard errors using delta method for a single survey, the 95\% confidence interval, and the logit of the estimates.
 #' @seealso \code{\link{getDirectList}}
+#' @author Zehang Richard Li, Bryan Martin, Laina Mercer
+#' @references Li, Z., Hsiao, Y., Godwin, J., Martin, B. D., Wakefield, J., Clark, S. J., & with support from the United Nations Inter-agency Group for Child Mortality Estimation and its technical advisory group. (2019). \emph{Changes in the spatial distribution of the under-five mortality rate: Small-area analysis of 122 DHS surveys in 262 subregions of 35 countries in Africa.} PloS one, 14(1), e0210645.
+#' @references Mercer, L. D., Wakefield, J., Pantazis, A., Lutambi, A. M., Masanja, H., & Clark, S. (2015). \emph{Space-time smoothing of complex survey data: small area estimation for child mortality.} The annals of applied statistics, 9(4), 1889.
 #' @examples
 #' \dontrun{
 #' data(DemoData)
@@ -35,10 +38,10 @@ getDirect <- function(births, years, regionVar = "region", timeVar = "time", clu
     if (!regionVar %in% colnames(births)) {
         if ("v101" %in% colnames(births)) {
             colnames(births)[which(colnames(births) == "v101")] <- regionVar
-            warning("region variable not defined: using v101", immediate. = TRUE)
+            message("region variable not defined: using v101")
         } else if ("v024" %in% colnames(births)) {
             colnames(births)[which(colnames(births) == "v024")] <- regionVar
-            warning("region variable not defined: using v024", immediate. = TRUE)
+            message("region variable not defined: using v024")
         } else {
             stop("region variable not defined, and no v101 or v024!")
         }
@@ -135,7 +138,7 @@ getDirect <- function(births, years, regionVar = "region", timeVar = "time", clu
         if (dim(tmp)[1] == 0) {
             return(rep(NA, 5))
         } else if (sum(tmp$variables$died) == 0) {
-            warning(paste0(which.area, " ", which.time, " has no death, set to NA\n"),immediate. = TRUE)
+            message(paste0(which.area, " ", which.time, " has no death, set to NA"))
             return(rep(NA, 5))
         } else if(length(unique(tmp$variables$age0)) > 1){
             if(is.null(Ntrials)){
@@ -147,13 +150,13 @@ getDirect <- function(births, years, regionVar = "region", timeVar = "time", clu
             if(dim(summary(glm.ob)$coef)[1] < length(labels)){
                 bins.nodata <- length(labels) - dim(summary(glm.ob)$coef)[1] 
                 if(bins.nodata >= length(ns)/2){
-                    warning(paste0(which.area, " ", which.time, " has no observation in more than half of the age bins, set to NA\n"),immediate. = TRUE)
+                    message(paste0(which.area, " ", which.time, " has no observation in more than half of the age bins, set to NA"))
                     return(rep(NA, 5))
                 }
                 # This can only happen for the last one or several bins, since person-month are cumulative
                 ns.comb <- ns
                 ns.comb[length(ns) - bins.nodata] <- sum(ns[c(length(ns) - bins.nodata) : length(ns)])
-                warning(paste0(which.area, " ", which.time, " has no observations in ", bins.nodata, " age bins. They are collapsed with previous bins\n"),immediate. = TRUE)
+                message(paste0(which.area, " ", which.time, " has no observations in ", bins.nodata, " age bins. They are collapsed with previous bins"))
                 return(get.est.withNA(glm.ob, labels, ns.comb))
             }
             return(get.est.withNA(glm.ob, labels, ns))
