@@ -1,12 +1,13 @@
 #' Fit space-time smoothing models for a generic outcome from complex surveys.
 #'
 #' This function calculates the direct estimates by region and fit a simple spatial smoothing model to the direct estimates adjusting for survey design.
-#' 
 #' Normal or binary variables are currently supported. For binary variables, the logit transformation is performed on the direct estimates of probabilities, and a Gaussian additive model is fitted on the logit scale using INLA.
 #' 
+#' The function \code{smoothSurvey} replaces the previous function name \code{fitGeneric} (before version 1.0.0).
+#' 
 #' @param data data frame with region and strata information.
-#' @param geo Geo file
-#' @param Amat Adjacency matrix for the regions
+#' @param geo Deprecated argument from early versions.
+#' @param Amat Adjacency matrix for the regions.
 #' @param X Covariate matrix with the first column being the region names. Currently only supporting static region-level covariates.
 #' @param responseType Type of the response variable, currently supports 'binary' (default with logit link function) or 'gaussian'. 
 #' @param responseVar the response variable
@@ -19,28 +20,30 @@
 #' @param pc.u.phi hyperparameter U for the PC prior on the mixture probability phi in BYM2 model.
 #' @param pc.alpha.phi hyperparameter alpha for the PC prior on the mixture probability phi in BYM2 model.
 #' @param CI the desired posterior credible interval to calculate
-#' @param FUN the function to transform the posterior draws. Default to be identify function for normal variable and inverse logit transformation for binomial variables
 #' @param formula a string of user-specified random effects model to be used in the INLA call
-#' @param  timeVar The variable indicating time period. If set to NULL then the temporal model and space-time interaction model are ignored.
+#' @param timeVar The variable indicating time period. If set to NULL then the temporal model and space-time interaction model are ignored.
 #' @param time.model the model for temporal trends and interactions. It can be either "rw1" or "rw2".
 #' @param type.st can take values 0 (no interaction), or 1 to 4, corresponding to the type I to IV space-time interaction.
+#' @param direct.est data frame of direct estimates, with column names of response and region specified by \code{responseVar}, \code{regionVar}, and \code{timeVar}.  When \code{direct.est} is specified, it overwrites the \code{data} input. 
+#' @param direct.est.var the column name corresponding to the variance of direct estimates
+#' @param ... additional arguments passed to \code{svydesign} function.
 #' 
 #' 
 #' @return \item{HT}{Direct estimates}
-#' \item{smooth}{Spatially smoothed estimates}
+#' \item{smooth}{Smoothed direct estimates}
 #' \item{fit}{a fitted INLA object}
-#' \item{geo}{input argument}
-#' \item{Amat}{input argument}
 #' \item{CI}{input argument}
+#' \item{Amat}{input argument}
 #' \item{responseType}{input argument}
-#' \item{FUN}{input argument}
-#' @seealso \code{\link{getDirectList}}, \code{\link{fitINLA}}
+#' \item{formula}{INLA formula}
+#' @seealso \code{\link{getDirectList}}, \code{\link{smoothDirect}}
 #' @importFrom stats median quantile sd var aggregate as.formula
+#' @author Zehang Richard Li 
 #' @examples
 #' \dontrun{
 #' data(DemoData2)
 #' data(DemoMap2)
-#' fit <- fitGeneric(data=DemoData2, geo=DemoMap2$geo, 
+#' fit0 <- smoothSurvey(data=DemoData2,  
 #' Amat=DemoMap2$Amat, responseType="binary", 
 #' responseVar="tobacco.use", strataVar="strata", 
 #' weightVar="weights", regionVar="region", 
@@ -48,32 +51,80 @@
 #' 
 #' # Example with region-level covariates
 #'  Xmat <- aggregate(age~region, data = DemoData2, FUN = mean)
-#'  fit <- fitGeneric(data=DemoData2, geo=DemoMap2$geo, 
+#'  fit1 <- smoothSurvey(data=DemoData2,  
 #'   Amat=DemoMap2$Amat, responseType="binary", 
 #'   X = Xmat,
 #'   responseVar="tobacco.use", strataVar="strata", 
 #'   weightVar="weights", regionVar="region", 
 #'   clusterVar = "~clustid+id", CI = 0.95)
+#' 
+#' # Example with using only direct estimates as input instead of the full data
+#' direct <- fit$HT[, c("region", "HT.est", "HT.var")]
+#' fit2 <- smoothSurvey(data=NULL, direct.est = direct, 
+#'                     Amat=DemoMap2$Amat, regionVar="region",
+#'                     responseVar="HT.est", direct.est.var = "HT.var", 
+#'                     responseType = "binary")
+#' # Check it is the same as fit0
+#' plot(fit2$smooth$mean, fit0$smooth$mean)
+#' 
+#' # Example with using only direct estimates as input, 
+#' #   and after transformation into a Gaussian smoothing model
+#' # Notice: the output are on the same scale as the input 
+#' #   and in this case, the logit estimates.    
+#' direct.logit <- fit$HT[, c("region", "HT.logit.est", "HT.logit.var")]
+#' fit3 <- smoothSurvey(data=NULL, direct.est = direct.logit, 
+#'                Amat=DemoMap2$Amat, regionVar="region",
+#'                responseVar="HT.logit.est", direct.est.var = "HT.logit.var",
+#'                responseType = "gaussian")
+#' # Check it is the same as fit0
+#' plot(fit3$smooth$mean, fit0$smooth$logit.mean)
+#' 
 #' }
 #' @export
 
 
-fitGeneric <- function(data, geo, Amat, X = NULL, responseType = c("binary", "gaussian")[1], responseVar, strataVar="strata", weightVar="weights", regionVar="region", clusterVar = "~v001+v002", pc.u = 1, pc.alpha = 0.01, pc.u.phi = 0.5, pc.alpha.phi = 2/3, CI = 0.95, FUN=NULL, formula = NULL, timeVar = NULL, time.model = c("rw1", "rw2")[1], type.st = 1){
+smoothSurvey <- function(data, geo = NULL, Amat, X = NULL, responseType = c("binary", "gaussian")[1], responseVar, strataVar="strata", weightVar="weights", regionVar="region", clusterVar = "~v001+v002", pc.u = 1, pc.alpha = 0.01, pc.u.phi = 0.5, pc.alpha.phi = 2/3, CI = 0.95, formula = NULL, timeVar = NULL, time.model = c("rw1", "rw2")[1], type.st = 1, direct.est = NULL, direct.est.var = NULL, ...){
 
     svy <- TRUE
-	if(!is.data.frame(data)){
-		stop("Input data needs to be a data frame")
-	}
-	if (is.null(strataVar)) {
-        message("Strata not defined. Ignoring sample design")
-        svy <- FALSE
-    }
     if(is.null(responseVar)){
-    	stop("Response variable not specified")
+        stop("Response variable not specified")
     }
     if(is.null(responseType)){
-    	stop("responseType not specified")
+            stop("responseType not specified")
+    }  
+    if(!is.null(direct.est)){
+        # TODO: this new input type later can be extended to Fay-Herriot
+        message("Using direct estimates as input instead of survey data.")
+        data <- direct.est
+        data$region0 <- direct.est[, regionVar]
+        data$response0 <- direct.est[, responseVar]
+        if(is.null(direct.est.var)){
+            stop("Need to specify column for the variance of direct estimates")
+        }
+        data$var0 <- direct.est[, direct.est.var]
+    }else{
+        if(!is.data.frame(data)){
+            stop("Input data needs to be a data frame")
+        }
+        if (is.null(strataVar)) {
+            message("Strata not defined. Ignoring sample design")
+            svy <- FALSE
+        }
+       
+        if (is.null(clusterVar)){
+            message("cluster not specified. Ignoring sample design")
+            svy <- FALSE
+        } 
+        data$response0 <- data[, responseVar]
+        data$region0 <- data[, regionVar]        
+        if(svy){
+            data$weights0 <- data[, weightVar]
+            data$strata0 <- data[, strataVar]
+        }
     }
+    if(!is.null(timeVar)) data$time0 <- data[, timeVar]     
+
+
     if(is.null(rownames(Amat))){
         stop("Row names of Amat needs to be specified to region names.")
     }
@@ -88,21 +139,18 @@ fitGeneric <- function(data, geo, Amat, X = NULL, responseType = c("binary", "ga
            sum(colnames(Amat) %in% X[,1] == FALSE) > 0) stop("Regions in the X matrix does not match the region names in Amat.")
     }
     
-    if (is.null(clusterVar)){
-        message("cluster not specified. Ignoring sample design")
-        svy <- FALSE
-    }
-    if(is.null(FUN)){
-        if(responseType == "binary"){
-            message("FUN is not specified, default to be expit()")
-            FUN <- expit
-        }else if(responseType == "gaussian"){
-            message("FUN is not specified, default to be no transformation")
-            FUN <- function(x){x}
-        }
+ 
+    # if(is.null(FUN)){
+    if(responseType == "binary"){
+        # message("FUN is not specified, default to be expit()")
+        FUN <- expit
+    }else if(responseType == "gaussian"){
+        # message("FUN is not specified, default to be no transformation")
+        FUN <- function(x){x}
     }
+    # }
     if (!isTRUE(requireNamespace("INLA", quietly = TRUE))) {
-    stop("You need to install the packages 'INLA'. Please run in your R terminal:\n install.packages('INLA', repos='https://www.math.ntnu.no/inla/R/stable')")
+    stop("You need to install the packages 'INLA'. Please run in your R terminal:\n  install.packages('INLA', repos=c(getOption('repos'), INLA='https://inla.r-inla-download.org/R/stable'), dep=TRUE)")
   }
   # If INLA is installed, then attach the Namespace (so that all the relevant functions are available)
   if (isTRUE(requireNamespace("INLA", quietly = TRUE))) {
@@ -110,41 +158,27 @@ fitGeneric <- function(data, geo, Amat, X = NULL, responseType = c("binary", "ga
       attachNamespace("INLA")
     }
   }
-
-
-    data$response0 <- data[, responseVar]
-    data$region0 <- data[, regionVar]        
-    if(svy){
-        data$weights0 <- data[, weightVar]
-        data$strata0 <- data[, strataVar]
-    }
-    if(!is.null(timeVar)) data$time0 <- data[, timeVar]
+  
 
     hyperpc1 <- list(prec = list(prior = "pc.prec", param = c(pc.u , pc.alpha)))
     hyperpc2 <- list(prec = list(prior = "pc.prec", param = c(pc.u , pc.alpha)),  phi = list(prior = 'pc', param = c(pc.u.phi , pc.alpha.phi)))
 
-
-    if(is.null(colnames(Amat)) || is.null(rownames(Amat))){
-        stop("column and row names for Amat needs to be specified to region names.")
-    }
-    if(sum(colnames(Amat) != rownames(Amat)) > 0){
-        stop("column names and row names do not agree in Amat.")
-    }
     if(sum(!data$region0 %in% colnames(Amat)) > 0){
        stop("Exist regions in data but not in the Amat.")
     }
 
-    if(svy){
+    if(svy && is.null(direct.est)){
         design <- survey::svydesign(
-        				ids = stats::formula(clusterVar),
+                        ids = stats::formula(clusterVar),
                         weights = ~weights0,
                         strata = ~strata0,
-                        data = data)
+                        data = data, 
+                        ...)
         if(!is.null(timeVar)){
-            mean <- survey::svyby(formula=~response0, by=~region0+time0, design=design, survey::svymean)
-            time.i <- mean$time0
+            mean <- survey::svyby(formula=~response0, by=~region0+time0, design=design, survey::svymean, drop.empty.groups=FALSE)
+            time.i <- as.numeric(as.character(mean$time0))
         }else{
-            mean <- survey::svyby(formula=~response0, by=~region0, design=design, survey::svymean)
+            mean <- survey::svyby(formula=~response0, by=~region0, design=design, survey::svymean, drop.empty.groups=FALSE)
         }
         name.i <- mean$region0            
         p.i <- mean$response0
@@ -161,11 +195,11 @@ fitGeneric <- function(data, geo, Amat, X = NULL, responseType = c("binary", "ga
             stop("responseType argument only supports binary or gaussian at the time.")
         }
         n <- y <- NA
-    }else{
+    }else if(is.null(direct.est)){
         if(!is.null(timeVar)){
             mean <- aggregate(response0 ~ region0+time0, data = data, FUN = function(x){c(mean(x), length(x), sum(x))})    
               name.i <- mean$region0
-              time.i <- mean$time0
+              time.i <- as.numeric(as.character(mean$time0))
               mean <- data.frame(mean[, -c(1:2)])
         }else{
             mean <- aggregate(response0 ~ region0, data = data, FUN = function(x){c(mean(x), length(x), sum(x))})
@@ -189,26 +223,46 @@ fitGeneric <- function(data, geo, Amat, X = NULL, responseType = c("binary", "ga
         }
         n <- mean$n
         y <- mean$y
+    }else{
+        p.i <- data$response0
+        var.i <- data$var0
+        if(responseType == "binary"){
+            ht <- log(p.i/(1-p.i))
+            ht.v <- var.i/(p.i^2*(1-p.i)^2)
+            ht.prec <- 1/ht.v
+        }else if(responseType == "gaussian"){
+            ht <- p.i
+            ht.v <- var.i
+            ht.prec <- 1/ht.v
+        }else{
+            stop("responseType argument only supports binary or gaussian at the time.")
+        } 
+        n <- NA
+        y <- NA
+        time.i <- as.numeric(as.character(data$time0))
+        name.i <- as.character(data$region0)
     }
 
-    dat <- data.frame(HT.est = ht, 
-                      HT.sd = ht.v ^ 0.5,
-                      HT.variance = ht.v,
-                      HT.prec = ht.prec,
-                      HT.est.original = p.i,
-                      HT.variance.original = var.i, 
+    dat <- data.frame(
+                      HT.est = p.i,
+                      HT.var = var.i, 
+                      HT.logit.est = ht,
+                      HT.logit.var = ht.v, 
+                      HT.logit.prec = ht.prec,  
                       n = n, 
-                      y = y)
+                      y = y)   
     if(!is.null(timeVar)){
         dat$time <- time.i
         dat$time.struct <- dat$time.unstruct <- time.int <- dat$time - min(dat$time) + 1
-    }
+    }   
     # make it consistent with map
     regnames <- as.character(name.i)
     # dat <- dat[match(rownames(Amat), regnames), ]
     dat$region <- as.character(name.i)
     dat$region.unstruct <- dat$region.struct <- dat$region.int <-  match(dat$region, rownames(Amat))
-    dat$HT.est[is.infinite(abs(dat$HT.est))] <- NA
+    dat$HT.logit.est[is.infinite(abs(dat$HT.logit.est))] <- NA  
+
+
     if(!is.null(timeVar)){
         dat <- dat[order(dat[, "time.struct"], dat[, "region.struct"]), ]
     }else{
@@ -219,7 +273,7 @@ fitGeneric <- function(data, geo, Amat, X = NULL, responseType = c("binary", "ga
     if(!svy && responseType == "binary"){
         formulatext <- "y ~ 1"
     }else{
-        formulatext <- "HT.est ~ 1"
+        formulatext <- "HT.logit.est ~ 1"
     }
     if(!is.null(X)){
         X <- data.frame(X)        
@@ -255,7 +309,7 @@ fitGeneric <- function(data, geo, Amat, X = NULL, responseType = c("binary", "ga
     }else if(!svy && responseType == "gaussian"){
         fit <- INLA::inla(formula, family="gaussian", control.compute = list(dic = T, mlik = T, cpo = T), data = dat, control.predictor = list(compute = TRUE), control.family = list(hyper= list(prec = list(initial= log(1), fixed= TRUE))), lincomb = NULL, quantiles = c((1-CI)/2, 0.5, 1-(1-CI)/2)) 
     }else{
-         fit <- INLA::inla(formula, family = "gaussian", control.compute = list(dic = T, mlik = T, cpo = T), data = dat, control.predictor = list(compute = TRUE), control.family = list(hyper= list(prec = list(initial= log(1), fixed= TRUE))), scale = dat$HT.prec,  lincomb = NULL, quantiles = c((1-CI)/2, 0.5, 1-(1-CI)/2)) 
+         fit <- INLA::inla(formula, family = "gaussian", control.compute = list(dic = T, mlik = T, cpo = T), data = dat, control.predictor = list(compute = TRUE), control.family = list(hyper= list(prec = list(initial= log(1), fixed= TRUE))), scale = dat$HT.logit.prec,  lincomb = NULL, quantiles = c((1-CI)/2, 0.5, 1-(1-CI)/2)) 
     }
     
 
@@ -266,45 +320,68 @@ fitGeneric <- function(data, geo, Amat, X = NULL, responseType = c("binary", "ga
         n <- n * max(dat$time.struct)
         temp <- dat$time[1:n]
     }
-    proj <- data.frame(region=dat$region[1:n], time = temp, mean=NA, variance=NA, median=NA, lower=NA, upper=NA, mean.original=NA, variance.original=NA, median.original=NA, lower.original=NA, upper.original=NA)
+    proj <- data.frame(region=dat$region[1:n], time = temp, mean=NA, var=NA, median=NA, lower=NA, upper=NA, logit.mean=NA, logit.var=NA, logit.median=NA, logit.lower=NA, logit.upper=NA)
     for(i in 1:n){
         tmp <- matrix(INLA::inla.rmarginal(1e5, fit$marginals.linear.predictor[[i]]))
         if(!svy && responseType == "binary"){
            tmp2 <- tmp
+            proj[i, "logit.mean"] <- mean(tmp2)
+            proj[i, "logit.var"] <- var(tmp2)
+            proj[i, "logit.lower"] <- quantile(tmp2, (1-CI)/2)
+            proj[i, "logit.upper"] <- quantile(tmp2, 1-(1-CI)/2)
+            proj[i, "logit.median"] <- median(tmp2)
+
+            proj[i, "mean"] <- fit$summary.fitted.values[i, "mean"]
+            proj[i, "var"] <- fit$summary.fitted.values[i, "sd"]^2
+            proj[i, "lower"] <- fit$summary.fitted.values[i, 3]
+            proj[i, "upper"] <- fit$summary.fitted.values[i, 5]
+            proj[i, "median"] <- fit$summary.fitted.values[i, "0.5quant"]  
+        }else{
+            tmp2 <- apply(tmp, 2, FUN)
             proj[i, "mean"] <- mean(tmp2)
-            proj[i, "variance"] <- var(tmp2)
+            proj[i, "var"] <- var(tmp2)
             proj[i, "lower"] <- quantile(tmp2, (1-CI)/2)
             proj[i, "upper"] <- quantile(tmp2, 1-(1-CI)/2)
             proj[i, "median"] <- median(tmp2)
 
-            proj[i, "mean.original"] <- fit$summary.fitted.values[i, "mean"]
-            proj[i, "variance.original"] <- fit$summary.fitted.values[i, "sd"]^2
-            proj[i, "lower.original"] <- fit$summary.fitted.values[i, 3]
-            proj[i, "upper.original"] <- fit$summary.fitted.values[i, 5]
-            proj[i, "median.original"] <- fit$summary.fitted.values[i, "0.5quant"]  
-        }else{
-           tmp2 <- apply(tmp, 2, FUN)
-            proj[i, "mean.original"] <- mean(tmp2)
-            proj[i, "variance.original"] <- var(tmp2)
-            proj[i, "lower.original"] <- quantile(tmp2, (1-CI)/2)
-            proj[i, "upper.original"] <- quantile(tmp2, 1-(1-CI)/2)
-            proj[i, "median.original"] <- median(tmp2)
-
-            proj[i, "mean"] <- fit$summary.fitted.values[i, "mean"]
-            proj[i, "variance"] <- fit$summary.fitted.values[i, "sd"]^2
-            proj[i, "lower"] <- fit$summary.fitted.values[i, 3]
-            proj[i, "upper"] <- fit$summary.fitted.values[i, 5]
-            proj[i, "median"] <- fit$summary.fitted.values[i, "0.5quant"]  
+            if(responseType == "binary"){
+                proj[i, "logit.mean"] <- fit$summary.fitted.values[i, "mean"]
+                proj[i, "logit.var"] <- fit$summary.fitted.values[i, "sd"]^2
+                proj[i, "logit.lower"] <- fit$summary.fitted.values[i, 3]
+                proj[i, "logit.upper"] <- fit$summary.fitted.values[i, 5]
+                proj[i, "logit.median"] <- fit$summary.fitted.values[i, "0.5quant"]  
+            }
         }
       
     }
-   return(list(HT = dat[, !colnames(dat) %in% c("region.struct", "region.unstruct", "region.int", "time.struct", "time.unstruct", "time.int")],
+
+   # organize output nicer 
+   HT <- dat[, !colnames(dat) %in% c("region.struct", "region.unstruct", "region.int", "time.struct", "time.unstruct", "time.int")]
+   HT <- cbind(region=HT[, "region"], HT[, colnames(HT) != "region"])
+   if("time" %in% names(HT)){
+       HT <- cbind(region=HT[, "region"], 
+                   time=HT[, "time"], 
+                   HT[, !(colnames(HT) %in% c("region", "time"))])
+   }
+   for(i in 1:dim(HT)[2]) HT[is.nan(HT[,i]), i] <- NA
+   if(sum(!is.na(HT$n)) == 0) HT <- HT[, colnames(HT) != "n"]
+   if(sum(!is.na(HT$y)) == 0) HT <- HT[, colnames(HT) != "y"]
+   if(sum(!is.na(proj$time)) == 0) proj <- proj[, colnames(proj) != "time"]
+   if(responseType == "gaussian"){
+        HT <- HT[, !colnames(HT) %in% c("HT.logit.est", "HT.logit.var", "HT.logit.prec")]
+        proj <- proj[, !colnames(HT) %in% c("logit.mean", "logit.var", "logit.median", "logit.lower", "logit.upper")]
+   }
+
+   return(list(HT = HT,
                smooth = proj, 
                fit = fit, 
                CI = CI,
                Amat = Amat,
-               geo = geo,
                responseType = responseType,
-               FUN = FUN, 
                formula = formula))
-}
\ No newline at end of file
+}
+
+
+#' @export
+#' @rdname smoothSurvey
+fitGeneric <-  smoothSurvey
\ No newline at end of file
